from django.contrib import admin
from django.contrib.admin import helpers
from django.contrib.admin.exceptions import DisallowedModelAdminToField
from django.contrib.admin.utils import (unquote,
                                        )
from django.core.exceptions import (PermissionDenied,
                                    )
from django.forms.formsets import all_valid

from django.utils.translation import gettext as _

IS_POPUP_VAR = '_popup'
TO_FIELD_VAR = '_to_field'

HORIZONTAL, VERTICAL = 1, 2


class MultipleChoiceAdmin(admin.ModelAdmin):
    # 多对多字段的值
    choice_field_value = {}

    def _changeform_view(self, request, object_id, form_url, extra_context):

        to_field = request.POST.get(TO_FIELD_VAR, request.GET.get(TO_FIELD_VAR))
        if to_field and not self.to_field_allowed(request, to_field):
            raise DisallowedModelAdminToField("The field %s cannot be referenced." % to_field)

        model = self.model
        opts = model._meta

        if request.method == 'POST' and '_saveasnew' in request.POST:
            object_id = None

        add = object_id is None

        if add:
            if not self.has_add_permission(request):
                raise PermissionDenied
            obj = None

        else:
            obj = self.get_object(request, unquote(object_id), to_field)

            if not self.has_change_permission(request, obj):
                raise PermissionDenied

            if obj is None:
                return self._get_obj_does_not_exist_redirect(request, opts, object_id)

        ModelForm = self.get_form(request, obj)
        if request.method == 'POST':
            form = ModelForm(request.POST, request.FILES, instance=obj)
            if form.is_valid():
                form_validated = True
                new_object = self.save_form(request, form, change=not add)
            else:
                form_validated = False
                new_object = form.instance
            formsets, inline_instances = self._create_formsets(request, new_object, change=not add)
            if all_valid(formsets) and form_validated:
                self.save_model(request, new_object, form, not add)
                self.save_related(request, form, formsets, not add)
                change_message = self.construct_change_message(request, form, formsets, add)
                if add:
                    self.log_addition(request, new_object, change_message)
                    return self.response_add(request, new_object)
                else:
                    self.log_change(request, new_object, change_message)
                    return self.response_change(request, new_object)
            else:
                form_validated = False
        else:
            if add:
                initial = self.get_changeform_initial_data(request)
                form = ModelForm(initial=initial)
                formsets, inline_instances = self._create_formsets(request, form.instance, change=False)
            else:
                form = ModelForm(instance=obj, initial=self.choice_field_value)
                formsets, inline_instances = self._create_formsets(request, obj, change=True)

        adminForm = helpers.AdminForm(
            form,
            list(self.get_fieldsets(request, obj)),
            self.get_prepopulated_fields(request, obj),
            self.get_readonly_fields(request, obj),
            model_admin=self)
        media = self.media + adminForm.media

        inline_formsets = self.get_inline_formsets(request, formsets, inline_instances, obj)
        for inline_formset in inline_formsets:
            media = media + inline_formset.media

        context = dict(
            self.admin_site.each_context(request),
            title=(_('Add %s') if add else _('Change %s')) % opts.verbose_name,
            adminform=adminForm,
            object_id=object_id,
            original=obj,
            is_popup=(IS_POPUP_VAR in request.POST or
                      IS_POPUP_VAR in request.GET),
            to_field=to_field,
            media=media,
            inline_admin_formsets=inline_formsets,
            errors=helpers.AdminErrorList(form, formsets),
            preserved_filters=self.get_preserved_filters(request),
        )

        # Hide the "Save" and "Save and continue" buttons if "Save as New" was
        # previously chosen to prevent the interface from getting confusing.
        if request.method == 'POST' and not form_validated and "_saveasnew" in request.POST:
            context['show_save'] = False
            context['show_save_and_continue'] = False
            # Use the change template instead of the add template.
            add = False

        context.update(extra_context or {})

        return self.render_change_form(request, context, add=add, change=not add, obj=obj, form_url=form_url)



from django import forms
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


class TagInput(forms.TextInput):
    class Media:
        js = []
        try:
            js += [
                settings.STATIC_URL + 'ktag/js/tagify.min.js',
            ]
        except AttributeError:
            print('eeerrr')
            raise ImproperlyConfigured("JS ")

    template_name = 'ktag/ktag_input.html'

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        context['widget']['type'] = self.input_type

        context['widget']['ktag_args'] = getattr(self, 'ktag_args', None) or {'placeholder': 'write some tags',
                                                                              'delimiters': ',',
                                                                              'whitelist': [],
                                                                              'suggestionsMinChars': 1,
                                                                              'blacklist': [],
                                                                              'maxTags': None}
        return context



from django import forms
from django.core import validators

from ktag.widgets import TagInput


class TagField(forms.CharField):
    widget = TagInput

    def __init__(self, *, place_holder='', delimiters=',', data_list=None,
                 suggestions_chars=1, black_list=None, max_tags=None,
                 max_length=None, min_length=None, strip=True, empty_value='', **kwargs):

        self.max_length = max_length
        self.min_length = min_length
        self.strip = strip
        self.empty_value = empty_value
        super().__init__(**kwargs)
        if min_length is not None:
            self.validators.append(validators.MinLengthValidator(int(min_length)))
        if max_length is not None:
            self.validators.append(validators.MaxLengthValidator(int(max_length)))
        self.validators.append(validators.ProhibitNullCharactersValidator())

        self.delimiters = delimiters
        ktag_args = {}
        ktag_args['placeholder'] = place_holder
        ktag_args['delimiters'] = delimiters
        ktag_args['whitelist'] = data_list if data_list else []
        ktag_args['suggestionsMinChars'] = suggestions_chars
        ktag_args['blacklist'] = black_list if black_list else []
        ktag_args['maxTags'] = max_tags

        setattr(self.widget, 'ktag_args', ktag_args)

    def to_python(self, value):
        value = super().to_python(value)
        return value.split(self.delimiters)



import sys

from setuptools import setup, find_packages

with open("README.rst", encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='django-ktag',
    version='1.0.2',
    description='django tag input field ',
    url='https://github.com/gojuukaze/django-ktag',
    author="gojuukaze",
    author_email="i@ikaze.uu.me",

    long_description=long_description,
    license="GUN V3.0",
    packages=find_packages(exclude=['django_ktag*', 'example*', ]),

    install_requires=[
        'Django>=2.0.0',
    ],
    python_requires='>=3',
    platforms=['Any'],
    include_package_data=True,

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Framework :: Django',
        'Environment :: Web Environment',
    ],
)
